# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...requests.loyalty_account import LoyaltyAccountParams
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.create_loyalty_account_response import CreateLoyaltyAccountResponse
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...requests.search_loyalty_accounts_request_loyalty_account_query import (
    SearchLoyaltyAccountsRequestLoyaltyAccountQueryParams,
)
from ...types.search_loyalty_accounts_response import SearchLoyaltyAccountsResponse
from ...types.get_loyalty_account_response import GetLoyaltyAccountResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...requests.loyalty_event_accumulate_points import LoyaltyEventAccumulatePointsParams
from ...types.accumulate_loyalty_points_response import AccumulateLoyaltyPointsResponse
from ...requests.loyalty_event_adjust_points import LoyaltyEventAdjustPointsParams
from ...types.adjust_loyalty_points_response import AdjustLoyaltyPointsResponse
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawAccountsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        loyalty_account: LoyaltyAccountParams,
        idempotency_key: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateLoyaltyAccountResponse]:
        """
        Creates a loyalty account. To create a loyalty account, you must provide the `program_id` and a `mapping` with the `phone_number` of the buyer.

        Parameters
        ----------
        loyalty_account : LoyaltyAccountParams
            The loyalty account to create.

        idempotency_key : str
            A unique string that identifies this `CreateLoyaltyAccount` request.
            Keys can be any valid string, but must be unique for every request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateLoyaltyAccountResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/loyalty/accounts",
            method="POST",
            json={
                "loyalty_account": convert_and_respect_annotation_metadata(
                    object_=loyalty_account, annotation=LoyaltyAccountParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateLoyaltyAccountResponse,
                    construct_type(
                        type_=CreateLoyaltyAccountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def search(
        self,
        *,
        query: typing.Optional[SearchLoyaltyAccountsRequestLoyaltyAccountQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SearchLoyaltyAccountsResponse]:
        """
        Searches for loyalty accounts in a loyalty program.

        You can search for a loyalty account using the phone number or customer ID associated with the account. To return all loyalty accounts, specify an empty `query` object or omit it entirely.

        Search results are sorted by `created_at` in ascending order.

        Parameters
        ----------
        query : typing.Optional[SearchLoyaltyAccountsRequestLoyaltyAccountQueryParams]
            The search criteria for the request.

        limit : typing.Optional[int]
            The maximum number of results to include in the response. The default value is 30.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to
            this endpoint. Provide this to retrieve the next set of
            results for the original query.

            For more information,
            see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SearchLoyaltyAccountsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/loyalty/accounts/search",
            method="POST",
            json={
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=SearchLoyaltyAccountsRequestLoyaltyAccountQueryParams, direction="write"
                ),
                "limit": limit,
                "cursor": cursor,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchLoyaltyAccountsResponse,
                    construct_type(
                        type_=SearchLoyaltyAccountsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, account_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetLoyaltyAccountResponse]:
        """
        Retrieves a loyalty account.

        Parameters
        ----------
        account_id : str
            The ID of the [loyalty account](entity:LoyaltyAccount) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetLoyaltyAccountResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/loyalty/accounts/{jsonable_encoder(account_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetLoyaltyAccountResponse,
                    construct_type(
                        type_=GetLoyaltyAccountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def accumulate_points(
        self,
        account_id: str,
        *,
        accumulate_points: LoyaltyEventAccumulatePointsParams,
        idempotency_key: str,
        location_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AccumulateLoyaltyPointsResponse]:
        """
        Adds points earned from a purchase to a [loyalty account](entity:LoyaltyAccount).

        - If you are using the Orders API to manage orders, provide the `order_id`. Square reads the order
        to compute the points earned from both the base loyalty program and an associated
        [loyalty promotion](entity:LoyaltyPromotion). For purchases that qualify for multiple accrual
        rules, Square computes points based on the accrual rule that grants the most points.
        For purchases that qualify for multiple promotions, Square computes points based on the most
        recently created promotion. A purchase must first qualify for program points to be eligible for promotion points.

        - If you are not using the Orders API to manage orders, provide `points` with the number of points to add.
        You must first perform a client-side computation of the points earned from the loyalty program and
        loyalty promotion. For spend-based and visit-based programs, you can call [CalculateLoyaltyPoints](api-endpoint:Loyalty-CalculateLoyaltyPoints)
        to compute the points earned from the base loyalty program. For information about computing points earned from a loyalty promotion, see
        [Calculating promotion points](https://developer.squareup.com/docs/loyalty-api/loyalty-promotions#calculate-promotion-points).

        Parameters
        ----------
        account_id : str
            The ID of the target [loyalty account](entity:LoyaltyAccount).

        accumulate_points : LoyaltyEventAccumulatePointsParams
            The points to add to the account.
            If you are using the Orders API to manage orders, specify the order ID.
            Otherwise, specify the points to add.

        idempotency_key : str
            A unique string that identifies the `AccumulateLoyaltyPoints` request.
            Keys can be any valid string but must be unique for every request.

        location_id : str
            The [location](entity:Location) where the purchase was made.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AccumulateLoyaltyPointsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/loyalty/accounts/{jsonable_encoder(account_id)}/accumulate",
            method="POST",
            json={
                "accumulate_points": convert_and_respect_annotation_metadata(
                    object_=accumulate_points, annotation=LoyaltyEventAccumulatePointsParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
                "location_id": location_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AccumulateLoyaltyPointsResponse,
                    construct_type(
                        type_=AccumulateLoyaltyPointsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def adjust(
        self,
        account_id: str,
        *,
        idempotency_key: str,
        adjust_points: LoyaltyEventAdjustPointsParams,
        allow_negative_balance: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AdjustLoyaltyPointsResponse]:
        """
        Adds points to or subtracts points from a buyer's account.

        Use this endpoint only when you need to manually adjust points. Otherwise, in your application flow, you call
        [AccumulateLoyaltyPoints](api-endpoint:Loyalty-AccumulateLoyaltyPoints)
        to add points when a buyer pays for the purchase.

        Parameters
        ----------
        account_id : str
            The ID of the target [loyalty account](entity:LoyaltyAccount).

        idempotency_key : str
            A unique string that identifies this `AdjustLoyaltyPoints` request.
            Keys can be any valid string, but must be unique for every request.

        adjust_points : LoyaltyEventAdjustPointsParams
            The points to add or subtract and the reason for the adjustment. To add points, specify a positive integer.
            To subtract points, specify a negative integer.

        allow_negative_balance : typing.Optional[bool]
            Indicates whether to allow a negative adjustment to result in a negative balance. If `true`, a negative
            balance is allowed when subtracting points. If `false`, Square returns a `BAD_REQUEST` error when subtracting
            the specified number of points would result in a negative balance. The default value is `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AdjustLoyaltyPointsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/loyalty/accounts/{jsonable_encoder(account_id)}/adjust",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "adjust_points": convert_and_respect_annotation_metadata(
                    object_=adjust_points, annotation=LoyaltyEventAdjustPointsParams, direction="write"
                ),
                "allow_negative_balance": allow_negative_balance,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AdjustLoyaltyPointsResponse,
                    construct_type(
                        type_=AdjustLoyaltyPointsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawAccountsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        loyalty_account: LoyaltyAccountParams,
        idempotency_key: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateLoyaltyAccountResponse]:
        """
        Creates a loyalty account. To create a loyalty account, you must provide the `program_id` and a `mapping` with the `phone_number` of the buyer.

        Parameters
        ----------
        loyalty_account : LoyaltyAccountParams
            The loyalty account to create.

        idempotency_key : str
            A unique string that identifies this `CreateLoyaltyAccount` request.
            Keys can be any valid string, but must be unique for every request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateLoyaltyAccountResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/loyalty/accounts",
            method="POST",
            json={
                "loyalty_account": convert_and_respect_annotation_metadata(
                    object_=loyalty_account, annotation=LoyaltyAccountParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateLoyaltyAccountResponse,
                    construct_type(
                        type_=CreateLoyaltyAccountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def search(
        self,
        *,
        query: typing.Optional[SearchLoyaltyAccountsRequestLoyaltyAccountQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SearchLoyaltyAccountsResponse]:
        """
        Searches for loyalty accounts in a loyalty program.

        You can search for a loyalty account using the phone number or customer ID associated with the account. To return all loyalty accounts, specify an empty `query` object or omit it entirely.

        Search results are sorted by `created_at` in ascending order.

        Parameters
        ----------
        query : typing.Optional[SearchLoyaltyAccountsRequestLoyaltyAccountQueryParams]
            The search criteria for the request.

        limit : typing.Optional[int]
            The maximum number of results to include in the response. The default value is 30.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to
            this endpoint. Provide this to retrieve the next set of
            results for the original query.

            For more information,
            see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SearchLoyaltyAccountsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/loyalty/accounts/search",
            method="POST",
            json={
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=SearchLoyaltyAccountsRequestLoyaltyAccountQueryParams, direction="write"
                ),
                "limit": limit,
                "cursor": cursor,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchLoyaltyAccountsResponse,
                    construct_type(
                        type_=SearchLoyaltyAccountsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, account_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetLoyaltyAccountResponse]:
        """
        Retrieves a loyalty account.

        Parameters
        ----------
        account_id : str
            The ID of the [loyalty account](entity:LoyaltyAccount) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetLoyaltyAccountResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/loyalty/accounts/{jsonable_encoder(account_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetLoyaltyAccountResponse,
                    construct_type(
                        type_=GetLoyaltyAccountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def accumulate_points(
        self,
        account_id: str,
        *,
        accumulate_points: LoyaltyEventAccumulatePointsParams,
        idempotency_key: str,
        location_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AccumulateLoyaltyPointsResponse]:
        """
        Adds points earned from a purchase to a [loyalty account](entity:LoyaltyAccount).

        - If you are using the Orders API to manage orders, provide the `order_id`. Square reads the order
        to compute the points earned from both the base loyalty program and an associated
        [loyalty promotion](entity:LoyaltyPromotion). For purchases that qualify for multiple accrual
        rules, Square computes points based on the accrual rule that grants the most points.
        For purchases that qualify for multiple promotions, Square computes points based on the most
        recently created promotion. A purchase must first qualify for program points to be eligible for promotion points.

        - If you are not using the Orders API to manage orders, provide `points` with the number of points to add.
        You must first perform a client-side computation of the points earned from the loyalty program and
        loyalty promotion. For spend-based and visit-based programs, you can call [CalculateLoyaltyPoints](api-endpoint:Loyalty-CalculateLoyaltyPoints)
        to compute the points earned from the base loyalty program. For information about computing points earned from a loyalty promotion, see
        [Calculating promotion points](https://developer.squareup.com/docs/loyalty-api/loyalty-promotions#calculate-promotion-points).

        Parameters
        ----------
        account_id : str
            The ID of the target [loyalty account](entity:LoyaltyAccount).

        accumulate_points : LoyaltyEventAccumulatePointsParams
            The points to add to the account.
            If you are using the Orders API to manage orders, specify the order ID.
            Otherwise, specify the points to add.

        idempotency_key : str
            A unique string that identifies the `AccumulateLoyaltyPoints` request.
            Keys can be any valid string but must be unique for every request.

        location_id : str
            The [location](entity:Location) where the purchase was made.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AccumulateLoyaltyPointsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/loyalty/accounts/{jsonable_encoder(account_id)}/accumulate",
            method="POST",
            json={
                "accumulate_points": convert_and_respect_annotation_metadata(
                    object_=accumulate_points, annotation=LoyaltyEventAccumulatePointsParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
                "location_id": location_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AccumulateLoyaltyPointsResponse,
                    construct_type(
                        type_=AccumulateLoyaltyPointsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def adjust(
        self,
        account_id: str,
        *,
        idempotency_key: str,
        adjust_points: LoyaltyEventAdjustPointsParams,
        allow_negative_balance: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AdjustLoyaltyPointsResponse]:
        """
        Adds points to or subtracts points from a buyer's account.

        Use this endpoint only when you need to manually adjust points. Otherwise, in your application flow, you call
        [AccumulateLoyaltyPoints](api-endpoint:Loyalty-AccumulateLoyaltyPoints)
        to add points when a buyer pays for the purchase.

        Parameters
        ----------
        account_id : str
            The ID of the target [loyalty account](entity:LoyaltyAccount).

        idempotency_key : str
            A unique string that identifies this `AdjustLoyaltyPoints` request.
            Keys can be any valid string, but must be unique for every request.

        adjust_points : LoyaltyEventAdjustPointsParams
            The points to add or subtract and the reason for the adjustment. To add points, specify a positive integer.
            To subtract points, specify a negative integer.

        allow_negative_balance : typing.Optional[bool]
            Indicates whether to allow a negative adjustment to result in a negative balance. If `true`, a negative
            balance is allowed when subtracting points. If `false`, Square returns a `BAD_REQUEST` error when subtracting
            the specified number of points would result in a negative balance. The default value is `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AdjustLoyaltyPointsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/loyalty/accounts/{jsonable_encoder(account_id)}/adjust",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "adjust_points": convert_and_respect_annotation_metadata(
                    object_=adjust_points, annotation=LoyaltyEventAdjustPointsParams, direction="write"
                ),
                "allow_negative_balance": allow_negative_balance,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AdjustLoyaltyPointsResponse,
                    construct_type(
                        type_=AdjustLoyaltyPointsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawCodesClient
from ...types.product_type import ProductType
from ...types.device_code_status import DeviceCodeStatus
from ...core.request_options import RequestOptions
from ...core.pagination import SyncPager
from ...types.device_code import DeviceCode
from ...types.list_device_codes_response import ListDeviceCodesResponse
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...requests.device_code import DeviceCodeParams
from ...types.create_device_code_response import CreateDeviceCodeResponse
from ...types.get_device_code_response import GetDeviceCodeResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawCodesClient
from ...core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CodesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCodesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCodesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCodesClient
        """
        return self._raw_client

    def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        location_id: typing.Optional[str] = None,
        product_type: typing.Optional[ProductType] = None,
        status: typing.Optional[DeviceCodeStatus] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[DeviceCode]:
        """
        Lists all DeviceCodes associated with the merchant.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for your original query.

            See [Paginating results](https://developer.squareup.com/docs/working-with-apis/pagination) for more information.

        location_id : typing.Optional[str]
            If specified, only returns DeviceCodes of the specified location.
            Returns DeviceCodes of all locations if empty.

        product_type : typing.Optional[ProductType]
            If specified, only returns DeviceCodes targeting the specified product type.
            Returns DeviceCodes of all product types if empty.

        status : typing.Optional[DeviceCodeStatus]
            If specified, returns DeviceCodes with the specified statuses.
            Returns DeviceCodes of status `PAIRED` and `UNPAIRED` if empty.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[DeviceCode]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.devices.codes.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/devices/codes",
            method="GET",
            params={
                "cursor": cursor,
                "location_id": location_id,
                "product_type": product_type,
                "status": status,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListDeviceCodesResponse,
                    construct_type(
                        type_=ListDeviceCodesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    location_id=location_id,
                    product_type=product_type,
                    status=status,
                    request_options=request_options,
                )
                _items = _parsed_response.device_codes
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        idempotency_key: str,
        device_code: DeviceCodeParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateDeviceCodeResponse:
        """
        Creates a DeviceCode that can be used to login to a Square Terminal device to enter the connected
        terminal mode.

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies this CreateDeviceCode request. Keys can
            be any valid string but must be unique for every CreateDeviceCode request.

            See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.

        device_code : DeviceCodeParams
            The device code to create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateDeviceCodeResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.devices.codes.create(
            idempotency_key="01bb00a6-0c86-4770-94ed-f5fca973cd56",
            device_code={
                "name": "Counter 1",
                "product_type": "TERMINAL_API",
                "location_id": "B5E4484SHHNYH",
            },
        )
        """
        response = self._raw_client.create(
            idempotency_key=idempotency_key, device_code=device_code, request_options=request_options
        )
        return response.data

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetDeviceCodeResponse:
        """
        Retrieves DeviceCode with the associated ID.

        Parameters
        ----------
        id : str
            The unique identifier for the device code.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetDeviceCodeResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.devices.codes.get(
            id="id",
        )
        """
        response = self._raw_client.get(id, request_options=request_options)
        return response.data


class AsyncCodesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCodesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCodesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCodesClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        location_id: typing.Optional[str] = None,
        product_type: typing.Optional[ProductType] = None,
        status: typing.Optional[DeviceCodeStatus] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[DeviceCode]:
        """
        Lists all DeviceCodes associated with the merchant.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for your original query.

            See [Paginating results](https://developer.squareup.com/docs/working-with-apis/pagination) for more information.

        location_id : typing.Optional[str]
            If specified, only returns DeviceCodes of the specified location.
            Returns DeviceCodes of all locations if empty.

        product_type : typing.Optional[ProductType]
            If specified, only returns DeviceCodes targeting the specified product type.
            Returns DeviceCodes of all product types if empty.

        status : typing.Optional[DeviceCodeStatus]
            If specified, returns DeviceCodes with the specified statuses.
            Returns DeviceCodes of status `PAIRED` and `UNPAIRED` if empty.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[DeviceCode]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.devices.codes.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/devices/codes",
            method="GET",
            params={
                "cursor": cursor,
                "location_id": location_id,
                "product_type": product_type,
                "status": status,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListDeviceCodesResponse,
                    construct_type(
                        type_=ListDeviceCodesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    location_id=location_id,
                    product_type=product_type,
                    status=status,
                    request_options=request_options,
                )
                _items = _parsed_response.device_codes
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        idempotency_key: str,
        device_code: DeviceCodeParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateDeviceCodeResponse:
        """
        Creates a DeviceCode that can be used to login to a Square Terminal device to enter the connected
        terminal mode.

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies this CreateDeviceCode request. Keys can
            be any valid string but must be unique for every CreateDeviceCode request.

            See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.

        device_code : DeviceCodeParams
            The device code to create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateDeviceCodeResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.devices.codes.create(
                idempotency_key="01bb00a6-0c86-4770-94ed-f5fca973cd56",
                device_code={
                    "name": "Counter 1",
                    "product_type": "TERMINAL_API",
                    "location_id": "B5E4484SHHNYH",
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            idempotency_key=idempotency_key, device_code=device_code, request_options=request_options
        )
        return response.data

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetDeviceCodeResponse:
        """
        Retrieves DeviceCode with the associated ID.

        Parameters
        ----------
        id : str
            The unique identifier for the device code.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetDeviceCodeResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.devices.codes.get(
                id="id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class LoyaltyEventDeleteReward(UncheckedBaseModel):
    """
    Provides metadata when the event `type` is `DELETE_REWARD`.
    """

    loyalty_program_id: str = pydantic.Field()
    """
    The ID of the [loyalty program](entity:LoyaltyProgram).
    """

    reward_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the deleted [loyalty reward](entity:LoyaltyReward).
    This field is returned only if the event source is `LOYALTY_API`.
    """

    points: int = pydantic.Field()
    """
    The number of points returned to the loyalty account.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .get_customer_response import GetCustomerResponse
import pydantic
from .error import Error
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BulkRetrieveCustomersResponse(UncheckedBaseModel):
    """
    Defines the fields included in the response body from the
    [BulkRetrieveCustomers](api-endpoint:Customers-BulkRetrieveCustomers) endpoint.
    """

    responses: typing.Optional[typing.Dict[str, GetCustomerResponse]] = pydantic.Field(default=None)
    """
    A map of responses that correspond to individual retrieve requests, represented by
    key-value pairs.
    
    Each key is the customer ID that was specified for a retrieve request and each value
    is the corresponding response.
    If the request succeeds, the value is the requested customer profile.
    If the request fails, the value contains any errors that occurred during the request.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any top-level errors that prevented the bulk operation from running.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
import typing_extensions
import typing_extensions
import typing
from .catalog_item_modifier_list_info import CatalogItemModifierListInfoParams
from ..types.catalog_item_product_type import CatalogItemProductType
from .catalog_item_option_for_item import CatalogItemOptionForItemParams
from .catalog_object_category import CatalogObjectCategoryParams
from .catalog_ecom_seo_data import CatalogEcomSeoDataParams
from .catalog_item_food_and_beverage_details import CatalogItemFoodAndBeverageDetailsParams
import typing

if typing.TYPE_CHECKING:
    from .catalog_object import CatalogObjectParams


class CatalogItemParams(typing_extensions.TypedDict):
    """
    A [CatalogObject](entity:CatalogObject) instance of the `ITEM` type, also referred to as an item, in the catalog.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The item's name. This is a searchable attribute for use in applicable query filters, its value must not be empty, and the length is of Unicode code points.
    """

    description: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The item's description. This is a searchable attribute for use in applicable query filters, and its value length is of Unicode code points.
    
    Deprecated at 2022-07-20, this field is planned to retire in 6 months. You should migrate to use `description_html` to set the description
    of the [CatalogItem](entity:CatalogItem) instance.  The `description` and `description_html` field values are kept in sync. If you try to
    set the both fields, the `description_html` text value overwrites the `description` value. Updates in one field are also reflected in the other,
    except for when you use an early version before Square API 2022-07-20 and `description_html` is set to blank, setting the `description` value to null
    does not nullify `description_html`.
    """

    abbreviation: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The text of the item's display label in the Square Point of Sale app. Only up to the first five characters of the string are used.
    This attribute is searchable, and its value length is of Unicode code points.
    """

    label_color: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The color of the item's display label in the Square Point of Sale app. This must be a valid hex color code.
    """

    is_taxable: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether the item is taxable (`true`) or non-taxable (`false`). Default is `true`.
    """

    category_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the item's category, if any. Deprecated since 2023-12-13. Use `CatalogItem.categories`, instead.
    """

    tax_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    A set of IDs indicating the taxes enabled for
    this item. When updating an item, any taxes listed here will be added to the item.
    Taxes may also be added to or deleted from an item using `UpdateItemTaxes`.
    """

    modifier_list_info: typing_extensions.NotRequired[
        typing.Optional[typing.Sequence[CatalogItemModifierListInfoParams]]
    ]
    """
    A set of `CatalogItemModifierListInfo` objects
    representing the modifier lists that apply to this item, along with the overrides and min
    and max limits that are specific to this item. Modifier lists
    may also be added to or deleted from an item using `UpdateItemModifierLists`.
    """

    variations: typing_extensions.NotRequired[typing.Optional[typing.Sequence["CatalogObjectParams"]]]
    """
    A list of [CatalogItemVariation](entity:CatalogItemVariation) objects for this item. An item must have
    at least one variation.
    """

    product_type: typing_extensions.NotRequired[CatalogItemProductType]
    """
    The product type of the item. Once set, the `product_type` value cannot be modified.
    
    Items of the `LEGACY_SQUARE_ONLINE_SERVICE` and `LEGACY_SQUARE_ONLINE_MEMBERSHIP` product types can be updated
    but cannot be created using the API.
    See [CatalogItemProductType](#type-catalogitemproducttype) for possible values
    """

    skip_modifier_screen: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    If `false`, the Square Point of Sale app will present the `CatalogItem`'s
    details screen immediately, allowing the merchant to choose `CatalogModifier`s
    before adding the item to the cart.  This is the default behavior.
    
    If `true`, the Square Point of Sale app will immediately add the item to the cart with the pre-selected
    modifiers, and merchants can edit modifiers by drilling down onto the item's details.
    
    Third-party clients are encouraged to implement similar behaviors.
    """

    item_options: typing_extensions.NotRequired[typing.Optional[typing.Sequence[CatalogItemOptionForItemParams]]]
    """
    List of item options IDs for this item. Used to manage and group item
    variations in a specified order.
    
    Maximum: 6 item options.
    """

    ecom_uri: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Deprecated. A URI pointing to a published e-commerce product page for the Item.
    """

    ecom_image_uris: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    Deprecated. A comma-separated list of encoded URIs pointing to a set of published e-commerce images for the Item.
    """

    image_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    The IDs of images associated with this `CatalogItem` instance.
    These images will be shown to customers in Square Online Store.
    The first image will show up as the icon for this item in POS.
    """

    sort_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A name to sort the item by. If this name is unspecified, namely, the `sort_name` field is absent, the regular `name` field is used for sorting.
    Its value must not be empty.
    
    It is currently supported for sellers of the Japanese locale only.
    """

    categories: typing_extensions.NotRequired[typing.Optional[typing.Sequence[CatalogObjectCategoryParams]]]
    """
    The list of categories.
    """

    description_html: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The item's description as expressed in valid HTML elements. The length of this field value, including those of HTML tags,
    is of Unicode points. With application query filters, the text values of the HTML elements and attributes are searchable. Invalid or
    unsupported HTML elements or attributes are ignored.
    
    Supported HTML elements include:
    - `a`: Link. Supports linking to website URLs, email address, and telephone numbers.
    - `b`, `strong`:  Bold text
    - `br`: Line break
    - `code`: Computer code
    - `div`: Section
    - `h1-h6`: Headings
    - `i`, `em`: Italics
    - `li`: List element
    - `ol`: Numbered list
    - `p`: Paragraph
    - `ul`: Bullet list
    - `u`: Underline
    
    
    Supported HTML attributes include:
    - `align`: Alignment of the text content
    - `href`: Link destination
    - `rel`: Relationship between link's target and source
    - `target`: Place to open the linked document
    """

    description_plaintext: typing_extensions.NotRequired[str]
    """
    A server-generated plaintext version of the `description_html` field, without formatting tags.
    """

    channels: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    A list of IDs representing channels, such as a Square Online site, where the item can be made visible or available.
    This field is read only and cannot be edited.
    """

    is_archived: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether this item is archived (`true`) or not (`false`).
    """

    ecom_seo_data: typing_extensions.NotRequired[CatalogEcomSeoDataParams]
    """
    The SEO data for a seller's Square Online store.
    """

    food_and_beverage_details: typing_extensions.NotRequired[CatalogItemFoodAndBeverageDetailsParams]
    """
    The food and beverage-specific details for the `FOOD_AND_BEV` item.
    """

    reporting_category: typing_extensions.NotRequired[CatalogObjectCategoryParams]
    """
    The item's reporting category.
    """

    is_alcoholic: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether this item is alcoholic (`true`) or not (`false`).
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.timecard_filter_status import TimecardFilterStatus
from .time_range import TimeRangeParams
from .timecard_workday import TimecardWorkdayParams


class TimecardFilterParams(typing_extensions.TypedDict):
    """
    Defines a filter used in a search for `Timecard` records. `AND` logic is
    used by Square's servers to apply each filter property specified.
    """

    location_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    Fetch timecards for the specified location.
    """

    status: typing_extensions.NotRequired[TimecardFilterStatus]
    """
    Fetch a `Timecard` instance by `Timecard.status`.
    See [TimecardFilterStatus](#type-timecardfilterstatus) for possible values
    """

    start: typing_extensions.NotRequired[TimeRangeParams]
    """
    Fetch `Timecard` instances that start in the time range - Inclusive.
    """

    end: typing_extensions.NotRequired[TimeRangeParams]
    """
    Fetch the `Timecard` instances that end in the time range - Inclusive.
    """

    workday: typing_extensions.NotRequired[TimecardWorkdayParams]
    """
    Fetch the `Timecard` instances based on the workday date range.
    """

    team_member_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    Fetch timecards for the specified team members.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .afterpay_details import AfterpayDetails
from .clearpay_details import ClearpayDetails
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BuyNowPayLaterDetails(UncheckedBaseModel):
    """
    Additional details about a Buy Now Pay Later payment type.
    """

    brand: typing.Optional[str] = pydantic.Field(default=None)
    """
    The brand used for the Buy Now Pay Later payment.
    The brand can be `AFTERPAY`, `CLEARPAY` or `UNKNOWN`.
    """

    afterpay_details: typing.Optional[AfterpayDetails] = pydantic.Field(default=None)
    """
    Details about an Afterpay payment. These details are only populated if the `brand` is
    `AFTERPAY`.
    """

    clearpay_details: typing.Optional[ClearpayDetails] = pydantic.Field(default=None)
    """
    Details about a Clearpay payment. These details are only populated if the `brand` is
    `CLEARPAY`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams


class BookingCustomAttributeDeleteResponseParams(typing_extensions.TypedDict):
    """
    Represents a response for an individual upsert request in a [BulkDeleteBookingCustomAttributes](api-endpoint:BookingCustomAttributes-BulkDeleteBookingCustomAttributes) operation.
    """

    booking_id: typing_extensions.NotRequired[str]
    """
    The ID of the [booking](entity:Booking) associated with the custom attribute.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred while processing the individual request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawActivitiesClient
from ...core.request_options import RequestOptions
from ...core.pagination import SyncPager
from ...types.gift_card_activity import GiftCardActivity
from ...types.list_gift_card_activities_response import ListGiftCardActivitiesResponse
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...requests.gift_card_activity import GiftCardActivityParams
from ...types.create_gift_card_activity_response import CreateGiftCardActivityResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawActivitiesClient
from ...core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ActivitiesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawActivitiesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawActivitiesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawActivitiesClient
        """
        return self._raw_client

    def list(
        self,
        *,
        gift_card_id: typing.Optional[str] = None,
        type: typing.Optional[str] = None,
        location_id: typing.Optional[str] = None,
        begin_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        sort_order: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[GiftCardActivity]:
        """
        Lists gift card activities. By default, you get gift card activities for all
        gift cards in the seller's account. You can optionally specify query parameters to
        filter the list. For example, you can get a list of gift card activities for a gift card,
        for all gift cards in a specific region, or for activities within a time window.

        Parameters
        ----------
        gift_card_id : typing.Optional[str]
            If a gift card ID is provided, the endpoint returns activities related
            to the specified gift card. Otherwise, the endpoint returns all gift card activities for
            the seller.

        type : typing.Optional[str]
            If a [type](entity:GiftCardActivityType) is provided, the endpoint returns gift card activities of the specified type.
            Otherwise, the endpoint returns all types of gift card activities.

        location_id : typing.Optional[str]
            If a location ID is provided, the endpoint returns gift card activities for the specified location.
            Otherwise, the endpoint returns gift card activities for all locations.

        begin_time : typing.Optional[str]
            The timestamp for the beginning of the reporting period, in RFC 3339 format.
            This start time is inclusive. The default value is the current time minus one year.

        end_time : typing.Optional[str]
            The timestamp for the end of the reporting period, in RFC 3339 format.
            This end time is inclusive. The default value is the current time.

        limit : typing.Optional[int]
            If a limit is provided, the endpoint returns the specified number
            of results (or fewer) per page. The maximum value is 100. The default value is 50.
            For more information, see [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination).

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.
            If a cursor is not provided, the endpoint returns the first page of the results.
            For more information, see [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination).

        sort_order : typing.Optional[str]
            The order in which the endpoint returns the activities, based on `created_at`.
            - `ASC` - Oldest to newest.
            - `DESC` - Newest to oldest (default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[GiftCardActivity]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.gift_cards.activities.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/gift-cards/activities",
            method="GET",
            params={
                "gift_card_id": gift_card_id,
                "type": type,
                "location_id": location_id,
                "begin_time": begin_time,
                "end_time": end_time,
                "limit": limit,
                "cursor": cursor,
                "sort_order": sort_order,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListGiftCardActivitiesResponse,
                    construct_type(
                        type_=ListGiftCardActivitiesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    gift_card_id=gift_card_id,
                    type=type,
                    location_id=location_id,
                    begin_time=begin_time,
                    end_time=end_time,
                    limit=limit,
                    cursor=_parsed_next,
                    sort_order=sort_order,
                    request_options=request_options,
                )
                _items = _parsed_response.gift_card_activities
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        idempotency_key: str,
        gift_card_activity: GiftCardActivityParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateGiftCardActivityResponse:
        """
        Creates a gift card activity to manage the balance or state of a [gift card](entity:GiftCard).
        For example, create an `ACTIVATE` activity to activate a gift card with an initial balance before first use.

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies the `CreateGiftCardActivity` request.

        gift_card_activity : GiftCardActivityParams
            The activity to create for the gift card. This activity must specify `gift_card_id` or `gift_card_gan` for the target
            gift card, the `location_id` where the activity occurred, and the activity `type` along with the corresponding activity details.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateGiftCardActivityResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.gift_cards.activities.create(
            idempotency_key="U16kfr-kA70er-q4Rsym-7U7NnY",
            gift_card_activity={
                "type": "ACTIVATE",
                "location_id": "81FN9BNFZTKS4",
                "gift_card_id": "gftc:6d55a72470d940c6ba09c0ab8ad08d20",
                "activate_activity_details": {
                    "order_id": "jJNGHm4gLI6XkFbwtiSLqK72KkAZY",
                    "line_item_uid": "eIWl7X0nMuO9Ewbh0ChIx",
                },
            },
        )
        """
        response = self._raw_client.create(
            idempotency_key=idempotency_key, gift_card_activity=gift_card_activity, request_options=request_options
        )
        return response.data


class AsyncActivitiesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawActivitiesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawActivitiesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawActivitiesClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        gift_card_id: typing.Optional[str] = None,
        type: typing.Optional[str] = None,
        location_id: typing.Optional[str] = None,
        begin_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        sort_order: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[GiftCardActivity]:
        """
        Lists gift card activities. By default, you get gift card activities for all
        gift cards in the seller's account. You can optionally specify query parameters to
        filter the list. For example, you can get a list of gift card activities for a gift card,
        for all gift cards in a specific region, or for activities within a time window.

        Parameters
        ----------
        gift_card_id : typing.Optional[str]
            If a gift card ID is provided, the endpoint returns activities related
            to the specified gift card. Otherwise, the endpoint returns all gift card activities for
            the seller.

        type : typing.Optional[str]
            If a [type](entity:GiftCardActivityType) is provided, the endpoint returns gift card activities of the specified type.
            Otherwise, the endpoint returns all types of gift card activities.

        location_id : typing.Optional[str]
            If a location ID is provided, the endpoint returns gift card activities for the specified location.
            Otherwise, the endpoint returns gift card activities for all locations.

        begin_time : typing.Optional[str]
            The timestamp for the beginning of the reporting period, in RFC 3339 format.
            This start time is inclusive. The default value is the current time minus one year.

        end_time : typing.Optional[str]
            The timestamp for the end of the reporting period, in RFC 3339 format.
            This end time is inclusive. The default value is the current time.

        limit : typing.Optional[int]
            If a limit is provided, the endpoint returns the specified number
            of results (or fewer) per page. The maximum value is 100. The default value is 50.
            For more information, see [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination).

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.
            If a cursor is not provided, the endpoint returns the first page of the results.
            For more information, see [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination).

        sort_order : typing.Optional[str]
            The order in which the endpoint returns the activities, based on `created_at`.
            - `ASC` - Oldest to newest.
            - `DESC` - Newest to oldest (default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[GiftCardActivity]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.gift_cards.activities.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/gift-cards/activities",
            method="GET",
            params={
                "gift_card_id": gift_card_id,
                "type": type,
                "location_id": location_id,
                "begin_time": begin_time,
                "end_time": end_time,
                "limit": limit,
                "cursor": cursor,
                "sort_order": sort_order,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListGiftCardActivitiesResponse,
                    construct_type(
                        type_=ListGiftCardActivitiesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    gift_card_id=gift_card_id,
                    type=type,
                    location_id=location_id,
                    begin_time=begin_time,
                    end_time=end_time,
                    limit=limit,
                    cursor=_parsed_next,
                    sort_order=sort_order,
                    request_options=request_options,
                )
                _items = _parsed_response.gift_card_activities
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        idempotency_key: str,
        gift_card_activity: GiftCardActivityParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateGiftCardActivityResponse:
        """
        Creates a gift card activity to manage the balance or state of a [gift card](entity:GiftCard).
        For example, create an `ACTIVATE` activity to activate a gift card with an initial balance before first use.

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies the `CreateGiftCardActivity` request.

        gift_card_activity : GiftCardActivityParams
            The activity to create for the gift card. This activity must specify `gift_card_id` or `gift_card_gan` for the target
            gift card, the `location_id` where the activity occurred, and the activity `type` along with the corresponding activity details.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateGiftCardActivityResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.gift_cards.activities.create(
                idempotency_key="U16kfr-kA70er-q4Rsym-7U7NnY",
                gift_card_activity={
                    "type": "ACTIVATE",
                    "location_id": "81FN9BNFZTKS4",
                    "gift_card_id": "gftc:6d55a72470d940c6ba09c0ab8ad08d20",
                    "activate_activity_details": {
                        "order_id": "jJNGHm4gLI6XkFbwtiSLqK72KkAZY",
                        "line_item_uid": "eIWl7X0nMuO9Ewbh0ChIx",
                    },
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            idempotency_key=idempotency_key, gift_card_activity=gift_card_activity, request_options=request_options
        )
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions


class BulkDeleteOrderCustomAttributesRequestDeleteCustomAttributeParams(typing_extensions.TypedDict):
    """
    Represents one delete within the bulk operation.
    """

    key: typing_extensions.NotRequired[str]
    """
    The key of the custom attribute to delete.  This key must match the key 
    of an existing custom attribute definition.
    """

    order_id: str
    """
    The ID of the target [order](entity:Order).
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions


class DeviceComponentDetailsCardReaderDetailsParams(typing_extensions.TypedDict):
    version: typing_extensions.NotRequired[str]
    """
    The version of the card reader.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .catalog_category import CatalogCategory
from .catalog_item import CatalogItem
from .catalog_item_option import CatalogItemOption
from .catalog_modifier_list import CatalogModifierList
from .catalog_object_category import CatalogObjectCategory
from .catalog_object_item import CatalogObjectItem
from .catalog_object_item_option import CatalogObjectItemOption
from .catalog_object_modifier_list import CatalogObjectModifierList
from .catalog_object_subscription_plan import CatalogObjectSubscriptionPlan
from .catalog_subscription_plan import CatalogSubscriptionPlan
import typing
from .error import Error
import pydantic
from .catalog_object import CatalogObject
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ListCatalogResponse(UncheckedBaseModel):
    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any errors that occurred during the request.
    """

    cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    The pagination cursor to be used in a subsequent request. If unset, this is the final response.
    See [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination) for more information.
    """

    objects: typing.Optional[typing.List[CatalogObject]] = pydantic.Field(default=None)
    """
    The CatalogObjects returned.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

OrderServiceChargeTreatmentType = typing.Union[
    typing.Literal["LINE_ITEM_TREATMENT", "APPORTIONED_TREATMENT"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.subscription_event_info_code import SubscriptionEventInfoCode


class SubscriptionEventInfoParams(typing_extensions.TypedDict):
    """
    Provides information about the subscription event.
    """

    detail: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A human-readable explanation for the event.
    """

    code: typing_extensions.NotRequired[SubscriptionEventInfoCode]
    """
    An info code indicating the subscription event that occurred.
    See [InfoCode](#type-infocode) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .payment import PaymentParams


class ListPaymentsResponseParams(typing_extensions.TypedDict):
    """
    Defines the response returned by [ListPayments](api-endpoint:Payments-ListPayments).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information about errors encountered during the request.
    """

    payments: typing_extensions.NotRequired[typing.Sequence[PaymentParams]]
    """
    The requested list of payments.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The pagination cursor to be used in a subsequent request. If empty,
    this is the final response.
    
    For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .vendor import VendorParams


class SearchVendorsResponseParams(typing_extensions.TypedDict):
    """
    Represents an output from a call to [SearchVendors](api-endpoint:Vendors-SearchVendors).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Errors encountered when the request fails.
    """

    vendors: typing_extensions.NotRequired[typing.Sequence[VendorParams]]
    """
    The [Vendor](entity:Vendor) objects matching the specified search filter.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The pagination cursor to be used in a subsequent request. If unset,
    this is the final response.
    
    See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawBreakTypesClient
from ...core.request_options import RequestOptions
from ...core.pagination import SyncPager
from ...types.break_type import BreakType
from ...types.list_break_types_response import ListBreakTypesResponse
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...requests.break_type import BreakTypeParams
from ...types.create_break_type_response import CreateBreakTypeResponse
from ...types.get_break_type_response import GetBreakTypeResponse
from ...types.update_break_type_response import UpdateBreakTypeResponse
from ...types.delete_break_type_response import DeleteBreakTypeResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawBreakTypesClient
from ...core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class BreakTypesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawBreakTypesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawBreakTypesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawBreakTypesClient
        """
        return self._raw_client

    def list(
        self,
        *,
        location_id: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[BreakType]:
        """
        Returns a paginated list of `BreakType` instances for a business.

        Parameters
        ----------
        location_id : typing.Optional[str]
            Filter the returned `BreakType` results to only those that are associated with the
            specified location.

        limit : typing.Optional[int]
            The maximum number of `BreakType` results to return per page. The number can range between 1
            and 200. The default is 200.

        cursor : typing.Optional[str]
            A pointer to the next page of `BreakType` results to fetch.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[BreakType]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.labor.break_types.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/labor/break-types",
            method="GET",
            params={
                "location_id": location_id,
                "limit": limit,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListBreakTypesResponse,
                    construct_type(
                        type_=ListBreakTypesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    location_id=location_id,
                    limit=limit,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.break_types
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        break_type: BreakTypeParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateBreakTypeResponse:
        """
        Creates a new `BreakType`.

        A `BreakType` is a template for creating `Break` objects.
        You must provide the following values in your request to this
        endpoint:

        - `location_id`
        - `break_name`
        - `expected_duration`
        - `is_paid`

        You can only have three `BreakType` instances per location. If you attempt to add a fourth
        `BreakType` for a location, an `INVALID_REQUEST_ERROR` "Exceeded limit of 3 breaks per location."
        is returned.

        Parameters
        ----------
        break_type : BreakTypeParams
            The `BreakType` to be created.

        idempotency_key : typing.Optional[str]
            A unique string value to ensure the idempotency of the operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateBreakTypeResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.labor.break_types.create(
            idempotency_key="PAD3NG5KSN2GL",
            break_type={
                "location_id": "CGJN03P1D08GF",
                "break_name": "Lunch Break",
                "expected_duration": "PT30M",
                "is_paid": True,
            },
        )
        """
        response = self._raw_client.create(
            break_type=break_type, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetBreakTypeResponse:
        """
        Returns a single `BreakType` specified by `id`.

        Parameters
        ----------
        id : str
            The UUID for the `BreakType` being retrieved.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBreakTypeResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.labor.break_types.get(
            id="id",
        )
        """
        response = self._raw_client.get(id, request_options=request_options)
        return response.data

    def update(
        self, id: str, *, break_type: BreakTypeParams, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdateBreakTypeResponse:
        """
        Updates an existing `BreakType`.

        Parameters
        ----------
        id : str
             The UUID for the `BreakType` being updated.

        break_type : BreakTypeParams
            The updated `BreakType`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateBreakTypeResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.labor.break_types.update(
            id="id",
            break_type={
                "location_id": "26M7H24AZ9N6R",
                "break_name": "Lunch",
                "expected_duration": "PT50M",
                "is_paid": True,
                "version": 1,
            },
        )
        """
        response = self._raw_client.update(id, break_type=break_type, request_options=request_options)
        return response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> DeleteBreakTypeResponse:
        """
        Deletes an existing `BreakType`.

        A `BreakType` can be deleted even if it is referenced from a `Shift`.

        Parameters
        ----------
        id : str
            The UUID for the `BreakType` being deleted.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteBreakTypeResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.labor.break_types.delete(
            id="id",
        )
        """
        response = self._raw_client.delete(id, request_options=request_options)
        return response.data


class AsyncBreakTypesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawBreakTypesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawBreakTypesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawBreakTypesClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        location_id: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[BreakType]:
        """
        Returns a paginated list of `BreakType` instances for a business.

        Parameters
        ----------
        location_id : typing.Optional[str]
            Filter the returned `BreakType` results to only those that are associated with the
            specified location.

        limit : typing.Optional[int]
            The maximum number of `BreakType` results to return per page. The number can range between 1
            and 200. The default is 200.

        cursor : typing.Optional[str]
            A pointer to the next page of `BreakType` results to fetch.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[BreakType]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.labor.break_types.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/labor/break-types",
            method="GET",
            params={
                "location_id": location_id,
                "limit": limit,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListBreakTypesResponse,
                    construct_type(
                        type_=ListBreakTypesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    location_id=location_id,
                    limit=limit,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.break_types
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        break_type: BreakTypeParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateBreakTypeResponse:
        """
        Creates a new `BreakType`.

        A `BreakType` is a template for creating `Break` objects.
        You must provide the following values in your request to this
        endpoint:

        - `location_id`
        - `break_name`
        - `expected_duration`
        - `is_paid`

        You can only have three `BreakType` instances per location. If you attempt to add a fourth
        `BreakType` for a location, an `INVALID_REQUEST_ERROR` "Exceeded limit of 3 breaks per location."
        is returned.

        Parameters
        ----------
        break_type : BreakTypeParams
            The `BreakType` to be created.

        idempotency_key : typing.Optional[str]
            A unique string value to ensure the idempotency of the operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateBreakTypeResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.labor.break_types.create(
                idempotency_key="PAD3NG5KSN2GL",
                break_type={
                    "location_id": "CGJN03P1D08GF",
                    "break_name": "Lunch Break",
                    "expected_duration": "PT30M",
                    "is_paid": True,
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            break_type=break_type, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetBreakTypeResponse:
        """
        Returns a single `BreakType` specified by `id`.

        Parameters
        ----------
        id : str
            The UUID for the `BreakType` being retrieved.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBreakTypeResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.labor.break_types.get(
                id="id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(id, request_options=request_options)
        return response.data

    async def update(
        self, id: str, *, break_type: BreakTypeParams, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdateBreakTypeResponse:
        """
        Updates an existing `BreakType`.

        Parameters
        ----------
        id : str
             The UUID for the `BreakType` being updated.

        break_type : BreakTypeParams
            The updated `BreakType`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateBreakTypeResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.labor.break_types.update(
                id="id",
                break_type={
                    "location_id": "26M7H24AZ9N6R",
                    "break_name": "Lunch",
                    "expected_duration": "PT50M",
                    "is_paid": True,
                    "version": 1,
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update(id, break_type=break_type, request_options=request_options)
        return response.data

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteBreakTypeResponse:
        """
        Deletes an existing `BreakType`.

        A `BreakType` can be deleted even if it is referenced from a `Shift`.

        Parameters
        ----------
        id : str
            The UUID for the `BreakType` being deleted.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteBreakTypeResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.labor.break_types.delete(
                id="id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete(id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .subscription_pricing_type import SubscriptionPricingType
import pydantic
from .money import Money
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SubscriptionPricing(UncheckedBaseModel):
    """
    Describes the pricing for the subscription.
    """

    type: typing.Optional[SubscriptionPricingType] = pydantic.Field(default=None)
    """
    RELATIVE or STATIC
    See [SubscriptionPricingType](#type-subscriptionpricingtype) for possible values
    """

    discount_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The ids of the discount catalog objects
    """

    price_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The price of the subscription, if STATIC
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class TimeRange(UncheckedBaseModel):
    """
    Represents a generic time range. The start and end values are
    represented in RFC 3339 format. Time ranges are customized to be
    inclusive or exclusive based on the needs of a particular endpoint.
    Refer to the relevant endpoint-specific documentation to determine
    how time ranges are handled.
    """

    start_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    A datetime value in RFC 3339 format indicating when the time range
    starts.
    """

    end_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    A datetime value in RFC 3339 format indicating when the time range
    ends.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .create_team_member_response import CreateTeamMemberResponseParams
from .error import ErrorParams


class BatchCreateTeamMembersResponseParams(typing_extensions.TypedDict):
    """
    Represents a response from a bulk create request containing the created `TeamMember` objects or error messages.
    """

    team_members: typing_extensions.NotRequired[typing.Dict[str, CreateTeamMemberResponseParams]]
    """
    The successfully created `TeamMember` objects. Each key is the `idempotency_key` that maps to the `CreateTeamMemberRequest`.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    The errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class PaymentBalanceActivityFeeDetailParams(typing_extensions.TypedDict):
    payment_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the payment associated with this activity
    This will only be populated when a principal LedgerEntryToken is also populated.
    If the fee is independent (there is no principal LedgerEntryToken) then this will likely not
    be populated.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .device_attributes_device_type import DeviceAttributesDeviceType
import pydantic
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class DeviceAttributes(UncheckedBaseModel):
    type: DeviceAttributesDeviceType = pydantic.Field(default="TERMINAL")
    """
    The device type.
    See [DeviceType](#type-devicetype) for possible values
    """

    manufacturer: str = pydantic.Field()
    """
    The maker of the device.
    """

    model: typing.Optional[str] = pydantic.Field(default=None)
    """
    The specific model of the device.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    A seller-specified name for the device.
    """

    manufacturers_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The manufacturer-supplied identifier for the device (where available). In many cases,
    this identifier will be a serial number.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The RFC 3339-formatted value of the most recent update to the device information.
    (Could represent any field update on the device.)
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The current version of software installed on the device.
    """

    merchant_token: typing.Optional[str] = pydantic.Field(default=None)
    """
    The merchant_token identifying the merchant controlling the device.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BulkDeleteMerchantCustomAttributesRequestMerchantCustomAttributeDeleteRequest(UncheckedBaseModel):
    """
    Represents an individual delete request in a [BulkDeleteMerchantCustomAttributes](api-endpoint:MerchantCustomAttributes-BulkDeleteMerchantCustomAttributes)
    request. An individual request contains an optional ID of the associated custom attribute definition
    and optional key of the associated custom attribute definition.
    """

    key: typing.Optional[str] = pydantic.Field(default=None)
    """
    The key of the associated custom attribute definition.
    Represented as a qualified key if the requesting app is not the definition owner.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .loyalty_program_status import LoyaltyProgramStatus
from .loyalty_program_reward_tier import LoyaltyProgramRewardTier
from .loyalty_program_expiration_policy import LoyaltyProgramExpirationPolicy
from .loyalty_program_terminology import LoyaltyProgramTerminology
from .loyalty_program_accrual_rule import LoyaltyProgramAccrualRule
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class LoyaltyProgram(UncheckedBaseModel):
    """
    Represents a Square loyalty program. Loyalty programs define how buyers can earn points and redeem points for rewards.
    Square sellers can have only one loyalty program, which is created and managed from the Seller Dashboard.
    For more information, see [Loyalty Program Overview](https://developer.squareup.com/docs/loyalty/overview).
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Square-assigned ID of the loyalty program. Updates to 
    the loyalty program do not modify the identifier.
    """

    status: typing.Optional[LoyaltyProgramStatus] = pydantic.Field(default=None)
    """
    Whether the program is currently active.
    See [LoyaltyProgramStatus](#type-loyaltyprogramstatus) for possible values
    """

    reward_tiers: typing.Optional[typing.List[LoyaltyProgramRewardTier]] = pydantic.Field(default=None)
    """
    The list of rewards for buyers, sorted by ascending points.
    """

    expiration_policy: typing.Optional[LoyaltyProgramExpirationPolicy] = pydantic.Field(default=None)
    """
    If present, details for how points expire.
    """

    terminology: typing.Optional[LoyaltyProgramTerminology] = pydantic.Field(default=None)
    """
    A cosmetic name for the “points” currency.
    """

    location_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The [locations](entity:Location) at which the program is active.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the program was created, in RFC 3339 format.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the reward was last updated, in RFC 3339 format.
    """

    accrual_rules: typing.Optional[typing.List[LoyaltyProgramAccrualRule]] = pydantic.Field(default=None)
    """
    Defines how buyers can earn loyalty points from the base loyalty program.
    To check for associated [loyalty promotions](entity:LoyaltyPromotion) that enable
    buyers to earn extra points, call [ListLoyaltyPromotions](api-endpoint:Loyalty-ListLoyaltyPromotions).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

MeasurementUnitTime = typing.Union[
    typing.Literal["GENERIC_MILLISECOND", "GENERIC_SECOND", "GENERIC_MINUTE", "GENERIC_HOUR", "GENERIC_DAY"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .money import MoneyParams
from ..types.order_service_charge_calculation_phase import OrderServiceChargeCalculationPhase
from .order_line_item_applied_tax import OrderLineItemAppliedTaxParams
from ..types.order_service_charge_treatment_type import OrderServiceChargeTreatmentType
from ..types.order_service_charge_scope import OrderServiceChargeScope


class OrderReturnServiceChargeParams(typing_extensions.TypedDict):
    """
    Represents the service charge applied to the original order.
    """

    uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A unique ID that identifies the return service charge only within this order.
    """

    source_service_charge_uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The service charge `uid` from the order containing the original
    service charge. `source_service_charge_uid` is `null` for
    unlinked returns.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of the service charge.
    """

    catalog_object_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The catalog object ID of the associated [OrderServiceCharge](entity:OrderServiceCharge).
    """

    catalog_version: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The version of the catalog object that this service charge references.
    """

    percentage: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The percentage of the service charge, as a string representation of
    a decimal number. For example, a value of `"7.25"` corresponds to a
    percentage of 7.25%.
    
    Either `percentage` or `amount_money` should be set, but not both.
    """

    amount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of a non-percentage-based service charge.
    
    Either `percentage` or `amount_money` should be set, but not both.
    """

    applied_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of money applied to the order by the service charge, including
    any inclusive tax amounts, as calculated by Square.
    
    - For fixed-amount service charges, `applied_money` is equal to `amount_money`.
    - For percentage-based service charges, `applied_money` is the money calculated using the percentage.
    """

    total_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total amount of money to collect for the service charge.
    
    __NOTE__: If an inclusive tax is applied to the service charge, `total_money`
    does not equal `applied_money` plus `total_tax_money` because the inclusive
    tax amount is already included in both `applied_money` and `total_tax_money`.
    """

    total_tax_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total amount of tax money to collect for the service charge.
    """

    calculation_phase: typing_extensions.NotRequired[OrderServiceChargeCalculationPhase]
    """
    The calculation phase after which to apply the service charge.
    See [OrderServiceChargeCalculationPhase](#type-orderservicechargecalculationphase) for possible values
    """

    taxable: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether the surcharge can be taxed. Service charges
    calculated in the `TOTAL_PHASE` cannot be marked as taxable.
    """

    applied_taxes: typing_extensions.NotRequired[typing.Optional[typing.Sequence[OrderLineItemAppliedTaxParams]]]
    """
    The list of references to `OrderReturnTax` entities applied to the
    `OrderReturnServiceCharge`. Each `OrderLineItemAppliedTax` has a `tax_uid`
    that references the `uid` of a top-level `OrderReturnTax` that is being
    applied to the `OrderReturnServiceCharge`. On reads, the applied amount is
    populated.
    """

    treatment_type: typing_extensions.NotRequired[OrderServiceChargeTreatmentType]
    """
    The treatment type of the service charge.
    See [OrderServiceChargeTreatmentType](#type-orderservicechargetreatmenttype) for possible values
    """

    scope: typing_extensions.NotRequired[OrderServiceChargeScope]
    """
    Indicates the level at which the apportioned service charge applies. For `ORDER`
    scoped service charges, Square generates references in `applied_service_charges` on
    all order line items that do not have them. For `LINE_ITEM` scoped service charges,
    the service charge only applies to line items with a service charge reference in their
    `applied_service_charges` field.
    
    This field is immutable. To change the scope of an apportioned service charge, you must delete
    the apportioned service charge and re-add it as a new apportioned service charge.
    See [OrderServiceChargeScope](#type-orderservicechargescope) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing

TenderBuyNowPayLaterDetailsStatus = typing.Union[
    typing.Literal["AUTHORIZED", "CAPTURED", "VOIDED", "FAILED"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.team_member_status import TeamMemberStatus


class SearchTeamMembersFilterParams(typing_extensions.TypedDict):
    """
    Represents a filter used in a search for `TeamMember` objects. `AND` logic is applied
    between the individual fields, and `OR` logic is applied within list-based fields.
    For example, setting this filter value:
    ```
    filter = (locations_ids = ["A", "B"], status = ACTIVE)
    ```
    returns only active team members assigned to either location "A" or "B".
    """

    location_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    When present, filters by team members assigned to the specified locations.
    When empty, includes team members assigned to any location.
    """

    status: typing_extensions.NotRequired[TeamMemberStatus]
    """
    When present, filters by team members who match the given status.
    When empty, includes team members of all statuses.
    See [TeamMemberStatus](#type-teammemberstatus) for possible values
    """

    is_owner: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    When present and set to true, returns the team member who is the owner of the Square account.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .timecard_filter import TimecardFilter
import pydantic
from .timecard_sort import TimecardSort
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class TimecardQuery(UncheckedBaseModel):
    """
    The parameters of a `Timecard` search query, which includes filter and sort options.
    """

    filter: typing.Optional[TimecardFilter] = pydantic.Field(default=None)
    """
    Query filter options.
    """

    sort: typing.Optional[TimecardSort] = pydantic.Field(default=None)
    """
    Sort order details.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .get_booking_response import GetBookingResponseParams
from .error import ErrorParams


class BulkRetrieveBookingsResponseParams(typing_extensions.TypedDict):
    """
    Response payload for bulk retrieval of bookings.
    """

    bookings: typing_extensions.NotRequired[typing.Dict[str, GetBookingResponseParams]]
    """
    Requested bookings returned as a map containing `booking_id` as the key and `RetrieveBookingResponse` as the value.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .team_member import TeamMemberParams


class CreateTeamMemberRequestParams(typing_extensions.TypedDict):
    """
    Represents a create request for a `TeamMember` object.
    """

    idempotency_key: typing_extensions.NotRequired[str]
    """
    A unique string that identifies this `CreateTeamMember` request.
    Keys can be any valid string, but must be unique for every request.
    For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).
    
    The minimum length is 1 and the maximum length is 45.
    """

    team_member: typing_extensions.NotRequired[TeamMemberParams]
    """
    **Required** The data used to create the `TeamMember` object. If you include `wage_setting`, you must provide
    `job_id` for each job assignment. To get job IDs, call [ListJobs](api-endpoint:Team-ListJobs).
    """



# This file was auto-generated by Fern from our API Definition.

import typing

OrderServiceChargeCalculationPhase = typing.Union[
    typing.Literal["SUBTOTAL_PHASE", "TOTAL_PHASE", "APPORTIONED_PERCENTAGE_PHASE", "APPORTIONED_AMOUNT_PHASE"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .destination_details_card_refund_details import DestinationDetailsCardRefundDetailsParams
from .destination_details_cash_refund_details import DestinationDetailsCashRefundDetailsParams
from .destination_details_external_refund_details import DestinationDetailsExternalRefundDetailsParams


class DestinationDetailsParams(typing_extensions.TypedDict):
    """
    Details about a refund's destination.
    """

    card_details: typing_extensions.NotRequired[DestinationDetailsCardRefundDetailsParams]
    """
    Details about a card refund. Only populated if the destination_type is `CARD`.
    """

    cash_details: typing_extensions.NotRequired[DestinationDetailsCashRefundDetailsParams]
    """
    Details about a cash refund. Only populated if the destination_type is `CASH`.
    """

    external_details: typing_extensions.NotRequired[DestinationDetailsExternalRefundDetailsParams]
    """
    Details about an external refund. Only populated if the destination_type is `EXTERNAL`.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .subscription import SubscriptionParams
from .subscription_action import SubscriptionActionParams


class CancelSubscriptionResponseParams(typing_extensions.TypedDict):
    """
    Defines output parameters in a response from the
    [CancelSubscription](api-endpoint:Subscriptions-CancelSubscription) endpoint.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Errors encountered during the request.
    """

    subscription: typing_extensions.NotRequired[SubscriptionParams]
    """
    The specified subscription scheduled for cancellation according to the action created by the request.
    """

    actions: typing_extensions.NotRequired[typing.Sequence[SubscriptionActionParams]]
    """
    A list of a single `CANCEL` action scheduled for the subscription.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions


class BulkPublishScheduledShiftsDataParams(typing_extensions.TypedDict):
    """
    Represents options for an individual publish request in a
    [BulkPublishScheduledShifts](api-endpoint:Labor-BulkPublishScheduledShifts)
    operation, provided as the value in a key-value pair.
    """

    version: typing_extensions.NotRequired[int]
    """
    The current version of the scheduled shift, used to enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
    control. If the provided version doesn't match the server version, the request fails.
    If omitted, Square executes a blind write, potentially overwriting data from another publish request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .customer_custom_attribute_filter import CustomerCustomAttributeFilterParams


class CustomerCustomAttributeFiltersParams(typing_extensions.TypedDict):
    """
    The custom attribute filters in a set of [customer filters](entity:CustomerFilter) used in a search query. Use this filter
    to search based on [custom attributes](entity:CustomAttribute) that are assigned to customer profiles. For more information, see
    [Search by custom attribute](https://developer.squareup.com/docs/customers-api/use-the-api/search-customers#search-by-custom-attribute).
    """

    filters: typing_extensions.NotRequired[typing.Optional[typing.Sequence[CustomerCustomAttributeFilterParams]]]
    """
    The custom attribute filters. Each filter must specify `key` and include the `filter` field with a type-specific filter,
    the `updated_at` field, or both. The provided keys must be unique within the list of custom attribute filters.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .order_entry import OrderEntry
import pydantic
from .order import Order
from .error import Error
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchOrdersResponse(UncheckedBaseModel):
    """
    Either the `order_entries` or `orders` field is set, depending on whether
    `return_entries` is set on the [SearchOrdersRequest](api-endpoint:Orders-SearchOrders).
    """

    order_entries: typing.Optional[typing.List[OrderEntry]] = pydantic.Field(default=None)
    """
    A list of [OrderEntries](entity:OrderEntry) that fit the query
    conditions. The list is populated only if `return_entries` is set to `true` in the request.
    """

    orders: typing.Optional[typing.List[Order]] = pydantic.Field(default=None)
    """
    A list of
    [Order](entity:Order) objects that match the query conditions. The list is populated only if
    `return_entries` is set to `false` in the request.
    """

    cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    The pagination cursor to be used in a subsequent request. If unset,
    this is the final response.
    For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    [Errors](entity:Error) encountered during the search.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class AcceptedPaymentMethodsParams(typing_extensions.TypedDict):
    apple_pay: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Whether Apple Pay is accepted at checkout.
    """

    google_pay: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Whether Google Pay is accepted at checkout.
    """

    cash_app_pay: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Whether Cash App Pay is accepted at checkout.
    """

    afterpay_clearpay: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Whether Afterpay/Clearpay is accepted at checkout.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Range(UncheckedBaseModel):
    """
    The range of a number value between the specified lower and upper bounds.
    """

    min: typing.Optional[str] = pydantic.Field(default=None)
    """
    The lower bound of the number range. At least one of `min` or `max` must be specified.
    If unspecified, the results will have no minimum value.
    """

    max: typing.Optional[str] = pydantic.Field(default=None)
    """
    The upper bound of the number range. At least one of `min` or `max` must be specified.
    If unspecified, the results will have no maximum value.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.order_line_item_discount_type import OrderLineItemDiscountType
from .money import MoneyParams
from ..types.order_line_item_discount_scope import OrderLineItemDiscountScope


class OrderReturnDiscountParams(typing_extensions.TypedDict):
    """
    Represents a discount being returned that applies to one or more return line items in an
    order.

    Fixed-amount, order-scoped discounts are distributed across all non-zero return line item totals.
    The amount distributed to each return line item is relative to that item’s contribution to the
    order subtotal.
    """

    uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A unique ID that identifies the returned discount only within this order.
    """

    source_discount_uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The discount `uid` from the order that contains the original application of this discount.
    """

    catalog_object_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The catalog object ID referencing [CatalogDiscount](entity:CatalogDiscount).
    """

    catalog_version: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The version of the catalog object that this discount references.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The discount's name.
    """

    type: typing_extensions.NotRequired[OrderLineItemDiscountType]
    """
    The type of the discount. If it is created by the API, it is `FIXED_PERCENTAGE` or `FIXED_AMOUNT`.
    
    Discounts that do not reference a catalog object ID must have a type of
    `FIXED_PERCENTAGE` or `FIXED_AMOUNT`.
    See [OrderLineItemDiscountType](#type-orderlineitemdiscounttype) for possible values
    """

    percentage: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The percentage of the tax, as a string representation of a decimal number.
    A value of `"7.25"` corresponds to a percentage of 7.25%.
    
    `percentage` is not set for amount-based discounts.
    """

    amount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total declared monetary amount of the discount.
    
    `amount_money` is not set for percentage-based discounts.
    """

    applied_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of discount actually applied to this line item. When an amount-based
    discount is at the order level, this value is different from `amount_money` because the discount
    is distributed across the line items.
    """

    scope: typing_extensions.NotRequired[OrderLineItemDiscountScope]
    """
    Indicates the level at which the `OrderReturnDiscount` applies. For `ORDER` scoped
    discounts, the server generates references in `applied_discounts` on all
    `OrderReturnLineItem`s. For `LINE_ITEM` scoped discounts, the discount is only applied to
    `OrderReturnLineItem`s with references in their `applied_discounts` field.
    See [OrderLineItemDiscountScope](#type-orderlineitemdiscountscope) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing

LoyaltyEventType = typing.Union[
    typing.Literal[
        "ACCUMULATE_POINTS",
        "CREATE_REWARD",
        "REDEEM_REWARD",
        "DELETE_REWARD",
        "ADJUST_POINTS",
        "EXPIRE_POINTS",
        "OTHER",
        "ACCUMULATE_PROMOTION_POINTS",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Snippet(UncheckedBaseModel):
    """
    Represents the snippet that is added to a Square Online site. The snippet code is injected into the `head` element of all pages on the site, except for checkout pages.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Square-assigned ID for the snippet.
    """

    site_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the site that contains the snippet.
    """

    content: str = pydantic.Field()
    """
    The snippet code, which can contain valid HTML, JavaScript, or both.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the snippet was initially added to the site, in RFC 3339 format.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the snippet was last updated on the site, in RFC 3339 format.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .application_type import ApplicationType
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class DeviceComponentDetailsApplicationDetails(UncheckedBaseModel):
    application_type: typing.Optional[ApplicationType] = pydantic.Field(default=None)
    """
    The type of application.
    See [ApplicationType](#type-applicationtype) for possible values
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The version of the application.
    """

    session_location: typing.Optional[str] = pydantic.Field(default=None)
    """
    The location_id of the session for the application.
    """

    device_code_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The id of the device code that was used to log in to the device.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .error import Error
import pydantic
from .delete_order_custom_attribute_response import DeleteOrderCustomAttributeResponse
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BulkDeleteOrderCustomAttributesResponse(UncheckedBaseModel):
    """
    Represents a response from deleting one or more order custom attributes.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any errors that occurred during the request.
    """

    values: typing.Dict[str, DeleteOrderCustomAttributeResponse] = pydantic.Field()
    """
     A map of responses that correspond to individual delete requests. Each response has the same ID 
    as the corresponding request and contains either a `custom_attribute` or an `errors` field.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

BusinessAppointmentSettingsCancellationPolicy = typing.Union[
    typing.Literal["CANCELLATION_TREATED_AS_NO_SHOW", "CUSTOM_POLICY"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .loyalty_event_filter import LoyaltyEventFilterParams


class LoyaltyEventQueryParams(typing_extensions.TypedDict):
    """
    Represents a query used to search for loyalty events.
    """

    filter: typing_extensions.NotRequired[LoyaltyEventFilterParams]
    """
    The query filter criteria.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .card import CardParams


class ListCardsResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields that are included in the response body of
    a request to the [ListCards](api-endpoint:Cards-ListCards) endpoint.

    Note: if there are errors processing the request, the card field will not be
    present.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information on errors encountered during the request.
    """

    cards: typing_extensions.NotRequired[typing.Sequence[CardParams]]
    """
    The requested list of `Card`s.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The pagination cursor to be used in a subsequent request. If empty,
    this is the final response.
    
    See [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination) for more information.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .money import Money
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class GiftCardActivityRefund(UncheckedBaseModel):
    """
    Represents details about a `REFUND` [gift card activity type](entity:GiftCardActivityType).
    """

    redeem_activity_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the refunded `REDEEM` gift card activity. Square populates this field if the 
    `payment_id` in the corresponding [RefundPayment](api-endpoint:Refunds-RefundPayment) request 
    represents a gift card redemption.
    
    For applications that use a custom payment processing system, this field is required when creating
    a `REFUND` activity. The provided `REDEEM` activity ID must be linked to the same gift card.
    """

    amount_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The amount added to the gift card for the refund. This value is a positive integer.
    
    This field is required when creating a `REFUND` activity. The amount can represent a full or partial refund.
    """

    reference_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    A client-specified ID that associates the gift card activity with an entity in another system.
    """

    payment_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the refunded payment. Square populates this field if the refund is for a 
    payment processed by Square. This field matches the `payment_id` in the corresponding
    [RefundPayment](api-endpoint:Refunds-RefundPayment) request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawEventsClient
from ..requests.search_events_query import SearchEventsQueryParams
from ..core.request_options import RequestOptions
from ..types.search_events_response import SearchEventsResponse
from ..types.disable_events_response import DisableEventsResponse
from ..types.enable_events_response import EnableEventsResponse
from ..types.list_event_types_response import ListEventTypesResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawEventsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class EventsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawEventsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawEventsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawEventsClient
        """
        return self._raw_client

    def search_events(
        self,
        *,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        query: typing.Optional[SearchEventsQueryParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchEventsResponse:
        """
        Search for Square API events that occur within a 28-day timeframe.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint. Provide this cursor to retrieve the next set of events for your original query.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        limit : typing.Optional[int]
            The maximum number of events to return in a single page. The response might contain fewer events. The default value is 100, which is also the maximum allowed value.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

            Default: 100

        query : typing.Optional[SearchEventsQueryParams]
            The filtering and sorting criteria for the search request. To retrieve additional pages using a cursor, you must use the original query.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchEventsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.events.search_events()
        """
        response = self._raw_client.search_events(
            cursor=cursor, limit=limit, query=query, request_options=request_options
        )
        return response.data

    def disable_events(self, *, request_options: typing.Optional[RequestOptions] = None) -> DisableEventsResponse:
        """
        Disables events to prevent them from being searchable.
        All events are disabled by default. You must enable events to make them searchable.
        Disabling events for a specific time period prevents them from being searchable, even if you re-enable them later.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DisableEventsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.events.disable_events()
        """
        response = self._raw_client.disable_events(request_options=request_options)
        return response.data

    def enable_events(self, *, request_options: typing.Optional[RequestOptions] = None) -> EnableEventsResponse:
        """
        Enables events to make them searchable. Only events that occur while in the enabled state are searchable.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EnableEventsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.events.enable_events()
        """
        response = self._raw_client.enable_events(request_options=request_options)
        return response.data

    def list_event_types(
        self, *, api_version: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> ListEventTypesResponse:
        """
        Lists all event types that you can subscribe to as webhooks or query using the Events API.

        Parameters
        ----------
        api_version : typing.Optional[str]
            The API version for which to list event types. Setting this field overrides the default version used by the application.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListEventTypesResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.events.list_event_types()
        """
        response = self._raw_client.list_event_types(api_version=api_version, request_options=request_options)
        return response.data


class AsyncEventsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawEventsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawEventsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawEventsClient
        """
        return self._raw_client

    async def search_events(
        self,
        *,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        query: typing.Optional[SearchEventsQueryParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchEventsResponse:
        """
        Search for Square API events that occur within a 28-day timeframe.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint. Provide this cursor to retrieve the next set of events for your original query.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        limit : typing.Optional[int]
            The maximum number of events to return in a single page. The response might contain fewer events. The default value is 100, which is also the maximum allowed value.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

            Default: 100

        query : typing.Optional[SearchEventsQueryParams]
            The filtering and sorting criteria for the search request. To retrieve additional pages using a cursor, you must use the original query.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchEventsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.events.search_events()


        asyncio.run(main())
        """
        response = await self._raw_client.search_events(
            cursor=cursor, limit=limit, query=query, request_options=request_options
        )
        return response.data

    async def disable_events(self, *, request_options: typing.Optional[RequestOptions] = None) -> DisableEventsResponse:
        """
        Disables events to prevent them from being searchable.
        All events are disabled by default. You must enable events to make them searchable.
        Disabling events for a specific time period prevents them from being searchable, even if you re-enable them later.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DisableEventsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.events.disable_events()


        asyncio.run(main())
        """
        response = await self._raw_client.disable_events(request_options=request_options)
        return response.data

    async def enable_events(self, *, request_options: typing.Optional[RequestOptions] = None) -> EnableEventsResponse:
        """
        Enables events to make them searchable. Only events that occur while in the enabled state are searchable.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EnableEventsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.events.enable_events()


        asyncio.run(main())
        """
        response = await self._raw_client.enable_events(request_options=request_options)
        return response.data

    async def list_event_types(
        self, *, api_version: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> ListEventTypesResponse:
        """
        Lists all event types that you can subscribe to as webhooks or query using the Events API.

        Parameters
        ----------
        api_version : typing.Optional[str]
            The API version for which to list event types. Setting this field overrides the default version used by the application.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListEventTypesResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.events.list_event_types()


        asyncio.run(main())
        """
        response = await self._raw_client.list_event_types(api_version=api_version, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawTeamClient
from ..core.request_options import RequestOptions
from ..types.list_jobs_response import ListJobsResponse
from ..requests.job import JobParams
from ..types.create_job_response import CreateJobResponse
from ..types.retrieve_job_response import RetrieveJobResponse
from ..types.update_job_response import UpdateJobResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawTeamClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TeamClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTeamClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTeamClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTeamClient
        """
        return self._raw_client

    def list_jobs(
        self, *, cursor: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> ListJobsResponse:
        """
        Lists jobs in a seller account. Results are sorted by title in ascending order.

        Parameters
        ----------
        cursor : typing.Optional[str]
            The pagination cursor returned by the previous call to this endpoint. Provide this
            cursor to retrieve the next page of results for your original request. For more information,
            see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListJobsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.team.list_jobs()
        """
        response = self._raw_client.list_jobs(cursor=cursor, request_options=request_options)
        return response.data

    def create_job(
        self, *, job: JobParams, idempotency_key: str, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateJobResponse:
        """
        Creates a job in a seller account. A job defines a title and tip eligibility. Note that
        compensation is defined in a [job assignment](entity:JobAssignment) in a team member's wage setting.

        Parameters
        ----------
        job : JobParams
            The job to create. The `title` field is required and `is_tip_eligible` defaults to true.

        idempotency_key : str
            A unique identifier for the `CreateJob` request. Keys can be any valid string,
            but must be unique for each request. For more information, see
            [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateJobResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.team.create_job(
            job={"title": "Cashier", "is_tip_eligible": True},
            idempotency_key="idempotency-key-0",
        )
        """
        response = self._raw_client.create_job(
            job=job, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    def retrieve_job(
        self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RetrieveJobResponse:
        """
        Retrieves a specified job.

        Parameters
        ----------
        job_id : str
            The ID of the job to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveJobResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.team.retrieve_job(
            job_id="job_id",
        )
        """
        response = self._raw_client.retrieve_job(job_id, request_options=request_options)
        return response.data

    def update_job(
        self, job_id: str, *, job: JobParams, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdateJobResponse:
        """
        Updates the title or tip eligibility of a job. Changes to the title propagate to all
        `JobAssignment`, `Shift`, and `TeamMemberWage` objects that reference the job ID. Changes to
        tip eligibility propagate to all `TeamMemberWage` objects that reference the job ID.

        Parameters
        ----------
        job_id : str
            The ID of the job to update.

        job : JobParams
            The job with the updated fields, either `title`, `is_tip_eligible`, or both. Only changed fields need
            to be included in the request. Optionally include `version` to enable optimistic concurrency control.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateJobResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.team.update_job(
            job_id="job_id",
            job={"title": "Cashier 1", "is_tip_eligible": True},
        )
        """
        response = self._raw_client.update_job(job_id, job=job, request_options=request_options)
        return response.data


class AsyncTeamClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTeamClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTeamClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTeamClient
        """
        return self._raw_client

    async def list_jobs(
        self, *, cursor: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> ListJobsResponse:
        """
        Lists jobs in a seller account. Results are sorted by title in ascending order.

        Parameters
        ----------
        cursor : typing.Optional[str]
            The pagination cursor returned by the previous call to this endpoint. Provide this
            cursor to retrieve the next page of results for your original request. For more information,
            see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListJobsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.team.list_jobs()


        asyncio.run(main())
        """
        response = await self._raw_client.list_jobs(cursor=cursor, request_options=request_options)
        return response.data

    async def create_job(
        self, *, job: JobParams, idempotency_key: str, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateJobResponse:
        """
        Creates a job in a seller account. A job defines a title and tip eligibility. Note that
        compensation is defined in a [job assignment](entity:JobAssignment) in a team member's wage setting.

        Parameters
        ----------
        job : JobParams
            The job to create. The `title` field is required and `is_tip_eligible` defaults to true.

        idempotency_key : str
            A unique identifier for the `CreateJob` request. Keys can be any valid string,
            but must be unique for each request. For more information, see
            [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateJobResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.team.create_job(
                job={"title": "Cashier", "is_tip_eligible": True},
                idempotency_key="idempotency-key-0",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create_job(
            job=job, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    async def retrieve_job(
        self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RetrieveJobResponse:
        """
        Retrieves a specified job.

        Parameters
        ----------
        job_id : str
            The ID of the job to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveJobResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.team.retrieve_job(
                job_id="job_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.retrieve_job(job_id, request_options=request_options)
        return response.data

    async def update_job(
        self, job_id: str, *, job: JobParams, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdateJobResponse:
        """
        Updates the title or tip eligibility of a job. Changes to the title propagate to all
        `JobAssignment`, `Shift`, and `TeamMemberWage` objects that reference the job ID. Changes to
        tip eligibility propagate to all `TeamMemberWage` objects that reference the job ID.

        Parameters
        ----------
        job_id : str
            The ID of the job to update.

        job : JobParams
            The job with the updated fields, either `title`, `is_tip_eligible`, or both. Only changed fields need
            to be included in the request. Optionally include `version` to enable optimistic concurrency control.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateJobResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.team.update_job(
                job_id="job_id",
                job={"title": "Cashier 1", "is_tip_eligible": True},
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update_job(job_id, job=job, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing

GiftCardActivityAdjustIncrementReason = typing.Union[
    typing.Literal["COMPLIMENTARY", "SUPPORT_ISSUE", "TRANSACTION_VOIDED"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .payment_link import PaymentLinkParams


class ListPaymentLinksResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Errors that occurred during the request.
    """

    payment_links: typing_extensions.NotRequired[typing.Sequence[PaymentLinkParams]]
    """
    The list of payment links.
    """

    cursor: typing_extensions.NotRequired[str]
    """
      When a response is truncated, it includes a cursor that you can use in a subsequent request
    to retrieve the next set of gift cards. If a cursor is not present, this is the final response.
    For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .error import Error
import pydantic
from .bank_account import BankAccount
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ListBankAccountsResponse(UncheckedBaseModel):
    """
    Response object returned by ListBankAccounts.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Information on errors encountered during the request.
    """

    bank_accounts: typing.Optional[typing.List[BankAccount]] = pydantic.Field(default=None)
    """
    List of BankAccounts associated with this account.
    """

    cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    When a response is truncated, it includes a cursor that you can 
    use in a subsequent request to fetch next set of bank accounts.
    If empty, this is the final response.
    
    For more information, see [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
